//------------------------------------------------------------------------------
/// @file  p9_sbe_chiplet_reset.H
///
/// @brief Identify all good chiplets excluding EQ/EC
/// Setup multicast groups for all chiplets
/// For all good chiplets excluding EQ/EC
/// For all enabled chiplets
// *!
// *! OWNER NAME  : Abhishek Agarwal  Email: abagarw8@in.ibm.com
// *! BACKUP NAME :                   Email:
//------------------------------------------------------------------------------
// *HWP HWP Owner   : Abhishek Agarwal <abagarw8@in.ibm.com>
// *HWP FW Owner    : Brian Silver <bsilver@us.ibm.com>
// *HWP Team        : Perv
// *HWP Level       : 1
// *HWP Consumed by : SBE
//------------------------------------------------------------------------------


#ifndef _P9_SBE_CHIPLET_RESET_H_
#define _P9_SBE_CHIPLET_RESET_H_


#include <fapi2.H>


typedef fapi2::ReturnCode (*p9_sbe_chiplet_reset_FP_t)(const
        fapi2::Target<fapi2::TARGET_TYPE_PROC_CHIP> &);

/// @brief Identify all good chiplets excluding EQ/EC
///      -- All chiplets will be reset and PLLs started
///      -- Partial bad - All nest Chiplets must be good, MC, IO can be partial bad
/// Setup multicast groups for all chiplets
///      -- Can't use the multicast for all non-nest chiplets
///      -- This is intended to be the eventual product setting
///      -- This includes the core/cache chiplets
/// For all good chiplets excluding EQ/EC
///      -- Setup Chiplet GP3 regs
///          -- Reset to default state
///          -- Set chiplet enable on all all good chiplets excluding EQ/EC
/// For all enabled chiplets
///      -- Start vital clocks and release endpoint reset
///      -- PCB Slave error register Reset
///
///
/// @param[in]     i_target_chip   Reference to TARGET_TYPE_PROC_CHIP target
/// @return  FAPI2_RC_SUCCESS if success, else error code.
extern "C"
{
    fapi2::ReturnCode p9_sbe_chiplet_reset(const
                                           fapi2::Target<fapi2::TARGET_TYPE_PROC_CHIP> & i_target_chip);
}

#endif
